#pragma once
/**
\file BMPacket.h

Class and convenience functions for dealing with Bitmap datapackets.

*/
#include "common.h"
#include "GridPacket.h"
#include "globals.h"

#include "color.h"

class HField;

/** reduced range RGB element.
*/
struct rgb_packed_data_element {
};

/** BMPacket is the specialized datatype used for passing RGB data around a world machine world.

	@ingroup data
*/
class BMPacket :
	public WMGridPacket<rgb96_data_element>
{
public:
	static BMPointer CreateEmpty();
	static BMPointer CreateNew(const PacketContext &context);
	BMPacket &operator=(const BMPacket &other);

	virtual int getType() { return WMP_rgb; };
	virtual char *getTypeName() { return "Bitmap"; };
	virtual bool queryType(const int type);	

	virtual dword Color() { return 0x00ff0000; };

	virtual PacketPointer Clone() const;			// Return a copy of this WMPacket-derived packet
	virtual bool canSerialize() { return true; };
	virtual bool Load(std::istream &input);		// Load from stream input
	virtual bool Save(std::ostream &output);	// Save a copy of this data to stream output

	void ClampRange();				/// Clamp data to lie 0..1.
/// Create greyscale RGB packet from a heightfield
	bool CreateFromHeightField(HFPointer other);
	virtual void Clear();

protected:
	friend class WMPacketFactory;
	BMPacket();									// Protect constructor/destructor
	BMPacket(const BMPacket &other);			// copy construction
	virtual ~BMPacket();
};

/// Safe Accessor inline function to provide a safe way to cast from WMPacket
/// in debug mode it will assert if the packet is not a BMPacket
/// @relates BMPacket
inline BMPointer BMP(PacketPointer const &pack) { 
	if (pack) {
		bool ok = pack->getType() == WMP_rgb;
		if (!ok)
			return NULL;
	}
	return BMPointer(static_cast<BMPacket*>(pack.get()) ); 
};

#define GetNewBMP(a)  (BMPacket::CreateNew(a))	
#define GetEmptyBMP() (BMPacket::CreateEmpty())
