#pragma once

namespace WorldMachine {
	enum ADVANCED_INTERFACE_TYPE { INTERFACE_BASE, INTERFACE_CONTAINER };

	class AdvancedDeviceInterface {
	public:
		virtual ADVANCED_INTERFACE_TYPE getInterfaceEnum() = 0;
	};
};


/** Interface Classes. These specify the class structure of the World Machine core without specifying implementation details. */
/*
class IStream {
	virtual void read(void *ptr, size_t bytes) = 0;
	virtual void write(void *ptr, size_t bytes) = 0;
};
*/
/** Interface Class for Device class. Added to WM 2.4 to provide better support for plugins that can work across interface versions.

	Primary constraint is that the ordering of the virtual functions cannot be changed, and every new release that changes this class must
	instead create a new Interface and rebase the internals to that class. In this way, plugins that are built against the old interface will still
	function.

	Note that Interfaces are NOT CURRENTLY USED

class IDevice {
public:
	virtual ~IDevice() = 0;
	virtual bool Load(IStream &in) = 0;
	virtual bool Save(IStream &out) = 0;

	virtual dword Color() =0; 
	virtual int GetWidth() = 0;
	virtual int GetHeight() =0;
	virtual char *GetDescription() =0;
	virtual void  SetName(const char *name) =0;
	virtual char *GetName() =0;
	virtual char *GetTypeName() =0;

	virtual char *GetInputName(int slot) =0;
	virtual char *GetOutputName(int slot) =0;

	virtual LayoutUIClient *GetLayoutUI() =0;

	virtual IPort *in(int slot = 0) =0;
	virtual IOutputPort *out(int slot = 0) =0;
	virtual IPort *parmin(int slot = 0) =0;
	virtual IPort *mask() =0;
	virtual IPort *coord_port() =0;


	virtual int  highest_input() =0;
	virtual int  highest_output() =0;
	virtual int  active_inputs() =0;
	virtual int  active_outputs() =0;
	virtual int  active_param_inputs() =0;	
	virtual int	 max_inputs() =0;
	virtual int  max_outputs() =0;
	virtual int  max_parm() =0;

	virtual void notifyPortConnected(int slot, int io, bool is_connecting) =0;	
	virtual bool RunUI() =0;
	virtual bool RunNameDialog() =0;
	virtual void RunHelpDialog() =0;

	virtual ACTIVATE_RESULT ActivateSequence(BuildContext &context) =0;
	virtual ACTIVATE_RESULT ActivateOnNeed(BuildContext &context) =0;
protected:
	virtual bool Activate(BuildContext &context) = 0;
public:
	virtual bool RunAuxAction(int index) =0;
	virtual int  getAuxActionSize() =0;
	virtual char *getAuxActionName(int index) =0;
	virtual DEV_DATA &GetLifeVars() =0;
	virtual int	Randomize(int newseed = 0) =0;	
	virtual int		GetI( const char *name) =0;
	virtual float	GetF( const char *name) =0;
	virtual bool	GetB(const char *name) =0;
	virtual int		&GetIRef( const char *name) =0;
	virtual float	&GetFRef( const char *name) =0;
	virtual bool	&GetBRef(const char *name) =0;
	virtual dword   &GetCRef(const char *name) =0;

	virtual ParameterSet &GetParams() =0;
	virtual void SetParams(ParameterSet &other) =0;
	virtual WMPacket *RetrieveParamPacket( int slot ) =0;
};

/*
class IDeviceWorld {
public:
	virtual IDevice *LoadDevice(std::istream &in) const =0;
	virtual int  Load(IStream &in) =0;
	virtual bool Save(IStream &out) =0;
	virtual bool Merge(IStream &in, std::vector<Device*> *merged_devices = NULL) =0;
	virtual void SetName(const char* _name) =0;
	virtual const char *GetName() =0;
	virtual void Clear() =0;
	virtual int  NumDevice() =0;
	virtual void AddDevice(IDevice *part, Coord &loc) =0;						
	virtual int  AddDeviceAtIndex(IDevice *part, Coord &loc, int index) =0;		
	virtual void DelDevice(Device *part) =0;
	virtual void DelDevice_StayLinked(IDevice *part) =0;
	virtual IDevice *GetDevice(int index) =0;
	virtual int FindDeviceIndex(IDevice *dev) =0;		
	virtual int GetDeviceIDNext() =0;

	virtual IDevice *Find(Coord &loc) =0;
	virtual IDevice *Find(int uid) =0;
	std::vector<Device*> Find(Rect &box) =0;
	std::vector<Device*> FindAllParts()  =0;
	std::vector<Device*> FindAllFamily(int type) =0;

	virtual bool CheckCycle() =0;
	virtual bool MakeLink(Port *P1, Port *P2) =0;
	virtual bool BreakLink(Port *P1, Port *P2 = NULL) =0;
	virtual bool BreakLink(Device *a, int port, int io) =0;
	virtual bool UnWireDevice(Device *part) =0;

	virtual bool AttachGraph(std::vector<Device*> &subparts);
	virtual std::vector<BrokenLink> DetachGraph(std::vector<Device*> &subparts, bool break_links = true);

	virtual int NumRoutes() =0;
	virtual WireRoute &GetWireRoute(int i) =0;
	virtual WireRoute *FindWireRoute(Port *a, Port *b) =0;
	virtual WireRoute &AddWireRoute(Port *a, Port *b, int coordinate, int location = -1) =0;
	virtual bool DeleteWireRoute(Port *a, Port *b) =0;


	int NumRoutePoints();							///< The number of routing points in the world
	int AddRoutePoint(Coord loc);					///< Add a new routing point and return its index value
	Coord GetRoutePoint(int index);					///< Get location of a given route point index
	bool isRoutePointValid(int index);				///< check if the route point is currently in use (valid)
	void ChangeRoutePoint(int index, Coord newloc);	///< Move a routing point. All wires that pass through that routing point will be moved as well
	void DeleteRoutePoint(int index, bool remove_others = true);	///< Remove a route point. If more than one route passes through and remove_others is false, it will remove the point from the first link but will not remove the route point..
	bool ChangeRouting(WireRoute &route, int routepoint, int new_routepoint);	///< Change a routepoint in a route from one to another. false if RP doesnt belong to the wire
	bool DeleteRouting(WireRoute &route, int start, int end, bool preserve_last);	///< Deletes route points between two routepoints. false if RPs don't belong to this wire
//@}

///@name  Device group ops
//@{
	/// Number of groups defined
	int numGroups() { return int(groups.size()); };
	/// Add a new group to the world
	void AddGroup(GroupStruct &newgroup) { groups.push_back(newgroup); };
	/// Remove a group
	void DelGroup(int i) { groups.erase(groups.begin() + i); };
	/// Retrieve group structure
	GroupStruct &GetGroup(int i) { return groups[i]; };

	int inGroup(Device *dev);	///< Return index of the group the device belongs to, or -1 if none
	int inGroup(Coord &loc);	///< Return index of the group the coordinate belongs to, or -1 if none
//@}
	void ResetMaps(PSTATE which = P_ALL);	///< Remove maps in the given PSTATE from devices in this world

	void setMemUseMode(int local_copy) { keeplocalcopy = local_copy; }; 
	int  getMemUseMode() { return keeplocalcopy; }; 

	int GetPacketMemory(int type);				///< Return use, in KB, of a packet of the given type at the current world size
	void RandomizeWorld();						///< Randommize any random generators or effects in this world	
	bool UpdateWorldOnEvent(int deviceevent);	///< Given a type of event, iterates all devices for that event and returns true if any devices were effected or updated by the event

	/// World State UID, used to track undo states,etc
	unsigned long long GetWorldUID() { return world_state_id; };
	/// We've taken an action that updates the world state
	void IncrementWorldUID() { world_state_id++; };


/// @name Render Results Management
//@{

	void ClearResults(PSTATE pstate);

	/// Flag device as unbuilt, and all of its dependents
	void FalsifyDevice(Device *dev);

	bool isDeviceBuilt(Device *dev, PSTATE state = P_PREVIEW);

	/// Return build result for the given device
	WMPacket *GetDefaultResult(Device *dev, PSTATE type);
	/// Return build result for a specific port
	WMPacket *GetPortResult(Device *dev, PSTATE type, int slot, int io);

	/// Apply a new set of results to the Device World
	/// calling with the defaults (or brs = NULL) will erase the corresponding resultset
	void ApplyResultSet(PSTATE type = P_ALL, BuildResultSet *brs = NULL);
	BuildResultSet *RetrieveResultSet(PSTATE type);

//@}

/// @name Render Extents Management
//@{
	PacketContext GetPacketContext();

	/// Retrieve the current render extents
	RenderExtent &GetCurrentRenderExtent();
	/// Retrieve index of the current render extent
	int GetCurrentRenderExtentIndex();

	/// Retrieve tiling information for current render extent
	TiledBuildData &GetTileData() { return GetCurrentRenderExtent().tbd; };
	/// Set tiling data for current RE
	void SetTileData(TiledBuildData &_td) { GetCurrentRenderExtent().tbd = _td; };

	/// Number of RE defined
	int numRenderExtent() { return int(extents.size()); };
	/// Retrieve a RE from given index
	RenderExtent &GetRenderExtent(int index);
	/// Activate a RE and make it the current one
	void ActivateRenderExtent(int index);	// Make an extent active
	/// Call to indicate to World that the current render extent has been updated externally
	void UpdatedRenderExtents();

	/// Add a RE to the world
	void AddRenderExtent(const RenderExtent &re);
	/// Remove a RE at the given index
	bool DelRenderExtent(int index);
//@}
	
	WorldCommonData &GetWorldParams()		{ return wdata; };
	void SetWorldParams(WorldCommonData &wd);
};
*/